<?php
namespace Teknisa\EST\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Model\Estoque\Entities\Requesto;
use Teknisa\Model\Estoque\Entities\Mensrequesto;
use Teknisa\Model\Estoque\Entities\Itrequesto;
use Teknisa\EST\Factories\Est33000RequisicaoEstoqueFactory;



class Est33000RequisicaoEstoqueService{

    protected $environment;
    protected $dataSourceManager;
    protected $newCode;
    protected $factory;

    public function __construct(Environment $environment, \Doctrine\ORM\EntityManager $entityManager ,$dataSourceManager,NewCode $newCode,
        Est33000RequisicaoEstoqueFactory $factory){
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
        $this->factory = $factory;
    }

    public function buscar($condicao){
        return $this->dataSourceManager->findBy($condicao);
    }

    public function deletar($dataSet){
        $this->dataSourceManager->delete($dataSet);
    }

    public function geraNovoCodigo($contador, $nrorg, $interval,$tamanho){
        return $this->newCode->getCodeByTableName($contador, $nrorg ,$interval,$tamanho);
    }

    public function deletarRequisicao ($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            //busca todoas  mensagens dos produtos da requisição selecionada para apagar
            $conditions = $this->condicaoProdutosRequisicao($row);
            $resultadoMensagem = $this->realizaConsultaBanco($conditions,'mensrequesto');
            $this->dataSourceManager->delete($resultadoMensagem);
            $this->entityManager->flush();

            //busca todos produtos da requisição selecionada para apagar
            $conditions = $this->condicaoProdutosRequisicao($row);
            $resultadoProdutos = $this->realizaConsultaBanco($conditions,'itrequesto');
            $this->dataSourceManager->delete($resultadoProdutos);
            $this->entityManager->flush();


            $array[] = $row;
            $dataSet = new DataSet('requesto', $array);
            $this->dataSourceManager->delete($dataSet);
            $this->entityManager->flush();

            $connection->commit();
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function deletarProduto ($row){
        try {
            //busca todos produtos da requisição selecionada
            $conditions = $this->condicaoProdutosRequisicao($row);
            $resultadoProdutos = $this->realizaConsultaBanco($conditions,'itrequesto');
            $produtos = $resultadoProdutos->getRows();

            // não é permitido deletar o produto se ele for o uníco cadastrado para a requisição.
            if(count($produtos) > 1 ){
                $connection = $this->entityManager->getConnection();
                $connection->beginTransaction();

                $this->deleteMensagemProduto($row);

                $array[] = $row;
                $dataSet = new DataSet('itrequesto', $array);
                $this->dataSourceManager->delete($dataSet);
                $this->entityManager->flush();

                $status = $this->atualizaSatusRequisicao($row);

                $connection->commit();
                return $status;
            }else{
                throw new \Exception("Não foi possível excluir. Este é o único produto cadastrado da requisição.", 1);
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteMensagemProduto ($row){
        try {
            $mensagem = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Mensrequesto')->findOneBy(
                array('nrorg' => $row['NRORG'], 'nrrequesto' => $row['NRREQUESTO'], 'cdfilial' => $row['CDFILIAL'], 'cdproduto' => $row['CDPRODUTO']    ));

            if($mensagem){
                $this->entityManager->remove($mensagem);
                $this->entityManager->flush();
            }
        }catch (\Exception $e) {
            throw $e;
        }
    }

    public function salvarNovaRequisicao($requisicao, $produto){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
          
            $nrrequesto = $this->saveRequisicao($requisicao[0]);
            $produto[0]['NRREQUESTO'] =$nrrequesto;
            $nrseqitem = $this->saveProduto($produto[0]);
            $dataSet = $this->atualizaSatusRequisicao($produto[0]);

            $resul = $dataSet->getRows();
            $status = [
                "idrequesto" => $resul['idrequesto'],
                "idstatusreq" => $resul['idstatusreq'],
                "nrrequesto" => $nrrequesto,
                "nrorg" =>$requisicao[0]['NRORG']
            ];
                
            $dados = new DataSet('requesto', $status);
            $connection->commit();
            return $dados;
        }catch (\Exception $e) {

            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function salvarNovoProduto($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            $nrseqitem = $this->saveProduto($row);
            $dataSet = $this->atualizaSatusRequisicao($row);

            $connection->commit();
            return $dataSet;
        }catch (\Exception $e) {

            $this->entityManager->rollback();
            throw $e;
        }
    }

    public function saveProduto($row) {
        //verifica se ja exite o um produto com mesmo lote e sublote cadastrado para a requisição
        $produto = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Itrequesto')->findOneBy(
            array(  'nrorg' => $row['NRORG'],
                    'nrrequesto' => $row['NRREQUESTO'],
                    'cdfilial' => $row['CDFILIAL'],
                    'cdproduto' => $row['CDPRODUTO'],
                    'nrloteestq'     => $row['NRLOTEESTQ'],
                    'nrsublote' => $row['NRSUBLOTE'],
                    'cdfilial' => $row['CDFILIAL']
            )
        );
        if(!$produto){
            $nrseqitem = $this->geraNovoCodigo('SEQITREQESTO'.$row['CDFILIAL'].$row['NRREQUESTO'].$row['CDPRODUTO'],$row['NRORG'],1,4);
            $row['NRSEQITEM'] = $nrseqitem;
            $row['NRSEQUITEM'] = null;
            $array[] = $row;
            $dataSet = new DataSet('itrequesto', $array);
            $this->dataSourceManager->persist($dataSet);
            $this->entityManager->flush();
            return $nrseqitem;
        }else{
            throw new \Exception("Produto já cadastrado.", 1);
        }
    }

    public function saveRequisicao($row){
        $nrrequesto = $this->geraNovoCodigo('NRREQUESTO',$row['NRORG'],1,10);
        $row['NRREQUESTO'] = $nrrequesto;

        $array[] = $row;
        $dataSet = new DataSet('requesto', $array);

        $this->dataSourceManager->persist($dataSet);
        $this->entityManager->flush();
        return $nrrequesto;
    }

    public function atualizaSatusRequisicao ($row){
        //busca todos produtos da requisição selecionada
        $conditions = $this->condicaoProdutosRequisicao($row);
        $resultadoProdutos = $this->realizaConsultaBanco($conditions,'itrequesto');
        $produtos = $resultadoProdutos->getRows();

        $situacaoProdutos = [];
        $idrequesto  = '';
        $idstatusreq = '';
        foreach ($produtos as &$p) {
            $situacaoProdutos[] = $p['IDSTITREQU'];
        }

        //se  algum item for difenre de  'A' IDREQUESTO = 'C'
        if ( in_array("E", $situacaoProdutos)) {
            $idrequesto = 'C';
        }else{
            $idrequesto = 'P';
        }

        if( count($situacaoProdutos) >1 ){
            $idstatusreq = 'P';
        }else if( in_array("E", $situacaoProdutos) ) {
            $idstatusreq = 'E';

        }else{
            $idstatusreq = 'A';
        }

        $requisição = $this->entityManager->getRepository('Teknisa\Model\Estoque\Entities\Requesto')->findOneBy(
            array('nrorg' => $row['NRORG'], 'nrrequesto' => $row['NRREQUESTO'], 'cdfilial' => $row['CDFILIAL'] ));

        $requisição->setIdrequesto($idrequesto);
        $requisição->setIdstatusreq($idstatusreq);
        $this->entityManager->persist($requisição);
        $this->entityManager->flush();

        $array = [
            "idrequesto" => $idrequesto,
            "idstatusreq" => $idstatusreq,
        ];
        $dataSet = new DataSet('requesto', $array);
        return $dataSet;
    }

    public function condicaoProdutosRequisicao($row) {
        $conditions = array(
          array(
            "value" => $this->environment->getNrOrgTrab(),
            "operator" => "=",
            "columnName" => "NRORG"
          ),
          array(
            "value" => $row['CDFILIAL'],
            "operator" => "=",
            "columnName" => "CDFILIAL"
          ),
          array(
            "value" => $row['NRREQUESTO'],
            "operator" => "=",
            "columnName" => "NRREQUESTO"
          )


        );
        return $conditions;
    }

    public function realizaConsultaBanco($conditions,$dataSourceName) {
        $filterCriteria = new FilterCriteria($dataSourceName,$conditions);
        $dataSet = $this->buscar($filterCriteria);
        return $dataSet;
    }
    
    public function updateProduto($row){
        try {    
            
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            if(($row['NRLOTEESTQ'] == "") && ($row['NRSUBLOTE'] == null)){
                $row['NRLOTEESTQ'] = null;
            }
            
            $result = $this->factory->updateProduto($row);
            
            if($result){
                $this->entityManager->commit();
            } else {
                throw new \Exception('Erro ao atualizar o Produto.');
            }
        } catch(\Exception $e) {
            $this->entityManager->rollback();
	        throw $e;
        }
    }

}