<?php
namespace Folha\Entities;

use HCM\Util\DateUtil;
use Importacao\Logic\Integracao\ProjetoImportacao;

class MigAgencia extends Base\MigAgencia
{

    /**
     * @param $cdagencia
     * @throws \Exception
     * @return \Folha\Entities\MigAgencia
     */
    public function setCdagencia ($cdagencia) {
        if ($cdagencia != '') {
            $cdagencia = preg_replace("/[^0-9]/","", $cdagencia); // somente numeros
            $cdagencia = \Importacao\Logic\Validacoes::verificaChaveDuplicada('AGENCIA', 'CDAGENCIA', ProjetoImportacao::getNrorg(), $cdagencia);
            $this->cdagencia = $cdagencia;
        } else {
            throw new \Exception("Agência: Não informada.");
        }
    }

    /**
     * @param $cdbanco
     * @throws \Exception
     * @return \Folha\Entities\MigAgencia
     */
    public function setCdbanco ($cdbanco) {
        if (empty($cdbanco)) {
            throw new \Exception("Banco: Não informado.");
        }

        $cdbanco = preg_replace("/[^0-9]/","", $cdbanco); // somente numeros
        $cdbanco = str_pad($cdbanco, 3, "0", STR_PAD_LEFT);
        $this->cdbanco = $cdbanco;
    }

    /**
     * @param $dtultatuagen
     * @return \Folha\Entities\MigAgencia|void
     * @throws \Exception
     */
    public function setDtultatuagen ($dtultatuagen) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtultatuagen)) {
            if (!empty($dtultatuagen)) {
                throw new \Exception("Dt. Ultima Agencia: Data '$dtultatuagen' com formato inválido ou em branco.");
            } else {
                throw new \Exception(null);
            }
        }

        if (!($dtultatuagen instanceof \DateTime)) {
            $dtultatuagen = DateUtil::getDataDeString($dtultatuagen);
        }

        $this->dtultatuagen = $dtultatuagen;
    }

    /**
     * @param $dtultatu
     * @throws \Exception
     * @return \Folha\Entities\MigAlteescala
     */
    public function setDtultatu ($dtultatu) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtultatu)) {
            throw new \Exception("Data '$dtultatu' com formato inválido ou em branco.");
        }

        if (!($dtultatu instanceof \DateTime)) {
            $dtultatu = DateUtil::getDataDeString($dtultatu);
        }

        $this->dtultatu = $dtultatu;
    }

    /**
     * @param $dtinclusao
     * @throws \Exception
     * @return \Folha\Entities\MigAlteescala
     */
    public function setDtinclusao ($dtinclusao) {
        if (!preg_match("/\d{2}\/\d{2}\/\d{4}/", $dtinclusao)) {
            throw new \Exception("Data '$dtinclusao' com formato inválido ou em branco.");
        }

        if (!($dtinclusao instanceof \DateTime)) {
            $dtinclusao = DateUtil::getDataDeString($dtinclusao);
        }

        $this->dtinclusao = $dtinclusao;
    }

}