/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: 8490_SUPSTP_MELHOR_PRECO_AUTO                                        */
/*                                                                            */
/* Descrio:                                                                 */
/*   Carrega os melhores preos para os produtos de uma cotao de um         */
/*   um fornecedor do tipo alternativo                                        */
/*                                                                            */
/* Criao:                                                                   */
/*    05/07/2000 - Adriano de Araujo                                          */
/*                                                                            */
/* Manutenes:                                                               */
/*   /  /   -                                                                 */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* @C:\WIN_PRJ\SQL\ORACLE\8490_SUPSTP_MELHOR_PRECO_AUTO.SQL                   */
/******************************************************************************/

CREATE OR REPLACE PROCEDURE MELHOR_PRECO_AUTO
                             (P_CDFILIAL    IN COLETAPR.CDFILIAL%TYPE,
                              P_NRCOLEPRECO IN COLETAPR.NRCOLEPRECO%TYPE,
                              P_DTINI       IN COLETAPR.DTINIVALCOL%TYPE,
                              P_DTFIN       IN COLETAPR.DTFINVALCOL%TYPE)
IS
V_CDOPERADOR COLETAPR.CDOPERADOR%TYPE;
V_ROWCOUNT  INT;
BEGIN
  V_ROWCOUNT := 0;
  SELECT
    CDOPERADOR INTO V_CDOPERADOR
  FROM
    COLETAPR
  WHERE
    CDFILIAL    = P_CDFILIAL    AND
    NRCOLEPRECO = P_NRCOLEPRECO;
  FOR C_MINPR IN
      (SELECT
         ITEMCOLE.CDPRODUTO,
         ITEMCOLE.CDREGIAO,
         MIN(ITEMCOLE.VRPRECUNIITM) AS VRPRECUNIITM
       FROM
         ITEMCOLE,
         FORNECEDOR
       WHERE
         ITEMCOLE.CDFILIAL     = P_CDFILIAL  AND
         ITEMCOLE.DTINIVALITM  >= P_DTINI    AND
         ITEMCOLE.DTFINVALITM  <= P_DTFIN    AND
         ITEMCOLE.CDFORNECED   = FORNECEDOR.CDFORNECED AND
         FORNECEDOR.IDTIPOFORN = 'N'
       GROUP BY
         ITEMCOLE.CDPRODUTO,
         ITEMCOLE.CDREGIAO )LOOP
    UPDATE ITEMCOLE
    SET
     ITEMCOLE.IDITMCOTPREC = 'S',
     ITEMCOLE.VRPRECUNIITM = C_MINPR.VRPRECUNIITM
    WHERE
      ITEMCOLE.CDFILIAL    = P_CDFILIAL    AND
      ITEMCOLE.NRCOLEPRECO = P_NRCOLEPRECO AND
      ITEMCOLE.CDPRODUTO  = C_MINPR.CDPRODUTO AND
      ITEMCOLE.CDREGIAO   = C_MINPR.CDREGIAO  AND
      EXISTS
       (SELECT
          PRODCOMP.CDPRODUTO
        FROM
          PRODCOMP, PRODUTO, PRODUTO PROD
        WHERE
          ITEMCOLE.CDPRODUTO = PROD.CDPRODUTO AND
          PRODCOMP.CDPRODUTO  = PRODUTO.CDPRODUTO AND
          PRODCOMP.CDOPERADOR = V_CDOPERADOR     AND
          PROD.CDARVPROD LIKE PRODUTO.CDARVPROD || '%') AND
      EXISTS
       (SELECT
          FORNECEDOR.CDFORNECED
        FROM
          FORNECEDOR
        WHERE
          ITEMCOLE.CDFORNECED  = FORNECEDOR.CDFORNECED AND
          FORNECEDOR.IDTIPOFORN = 'A');
      V_ROWCOUNT := SQL%ROWCOUNT + V_ROWCOUNT;
  END LOOP;
  IF V_ROWCOUNT > 0 THEN
    UPDATE COLETAPR
    SET IDTIPCOLREG = 'G'
    WHERE
      CDFILIAL    = P_CDFILIAL AND
      NRCOLEPRECO = P_NRCOLEPRECO;
  END IF;
END;
/

